import plutoprint
import pytest

def test_units():
    assert isinstance(plutoprint.UNITS_PT, float)
    assert isinstance(plutoprint.UNITS_PC, float)
    assert isinstance(plutoprint.UNITS_IN, float)
    assert isinstance(plutoprint.UNITS_PX, float)
    assert isinstance(plutoprint.UNITS_CM, float)
    assert isinstance(plutoprint.UNITS_MM, float)

    assert plutoprint.UNITS_PT == pytest.approx(1.0)
    assert plutoprint.UNITS_PC == pytest.approx(12.0)
    assert plutoprint.UNITS_IN == pytest.approx(72.0)
    assert plutoprint.UNITS_PX == pytest.approx(72.0 / 96.0)
    assert plutoprint.UNITS_CM == pytest.approx(72.0 / 2.54)
    assert plutoprint.UNITS_MM == pytest.approx(72.0 / 25.4)



import plutoprint
import argparse
import sys
import re

from datetime import datetime

PAGE_SIZES = {
    'a3': plutoprint.PAGE_SIZE_A3,
    'a4': plutoprint.PAGE_SIZE_A4,
    'a5': plutoprint.PAGE_SIZE_A5,
    'b4': plutoprint.PAGE_SIZE_B4,
    'b5': plutoprint.PAGE_SIZE_B5,
    'letter': plutoprint.PAGE_SIZE_LETTER,
    'legal': plutoprint.PAGE_SIZE_LEGAL,
    'ledger': plutoprint.PAGE_SIZE_LEDGER
}

UNITS_MAP = {
    'pt': plutoprint.UNITS_PT,
    'pc': plutoprint.UNITS_PC,
    'in': plutoprint.UNITS_IN,
    'cm': plutoprint.UNITS_CM,
    'mm': plutoprint.UNITS_MM,
    'px': plutoprint.UNITS_PX
}

METADATA_MAP = {
    'title': plutoprint.PDF_METADATA_TITLE,
    'author': plutoprint.PDF_METADATA_AUTHOR,
    'subject': plutoprint.PDF_METADATA_SUBJECT,
    'keywords': plutoprint.PDF_METADATA_KEYWORDS,
    'creator': plutoprint.PDF_METADATA_CREATOR,
}

class InfoAction(argparse.Action):
    def __call__(self, *args, **kwargs):
        print(plutoprint.__build_info__, end='')
        sys.exit()

def length(value):
    match = re.fullmatch(r'(\d+(?:.\d+)?)(pt|pc|in|cm|mm|px)', value.lower())
    if not match:
        raise argparse.ArgumentTypeError(f"invalid length value: '{value}'")
    num, unit = match.groups()
    return float(num) * UNITS_MAP[unit]

def main():
    parser = argparse.ArgumentParser(prog='plutoprint', description='Convert HTML to PDF.')

    parser.add_argument('input', help='Specify the input HTML filename or URL.')
    parser.add_argument('output', help='Specify the output PDF filename.')

    parser.add_argument('--size', type=str.lower, choices=PAGE_SIZES, help='Specify the page size (eg. A4).')
    parser.add_argument('--margin', type=length, metavar='MARGIN', help='Specify the page margin (eg. 72pt).')
    parser.add_argument('--media', type=str.lower, choices=['print', 'screen'], help='Specify the media type (eg. print, screen).')
    parser.add_argument('--orientation', type=str.lower, choices=['portrait', 'landscape'], help='Specify the page orientation (eg. portrait, landscape).')

    parser.add_argument('--margin-top', type=length, metavar='MARGIN', help='Specify the page margin top (eg. 72pt).')
    parser.add_argument('--margin-right', type=length, metavar='MARGIN', help='Specify the page margin right (eg. 72pt).')
    parser.add_argument('--margin-bottom', type=length, metavar='MARGIN', help='Specify the page margin bottom (eg. 72pt).')
    parser.add_argument('--margin-left', type=length, metavar='MARGIN', help='Specify the page margin left (eg. 72pt).')

    parser.add_argument('--width', type=length, help='Specify the page width (eg. 210mm).')
    parser.add_argument('--height', type=length, help='Specify the page height (eg. 297mm).')

    parser.add_argument('--page-start', type=int, default=plutoprint.MIN_PAGE_COUNT, metavar='START', help='Specify the starting page number to include in the output PDF.')
    parser.add_argument('--page-end', type=int, default=plutoprint.MAX_PAGE_COUNT, metavar='END', help='Specify the ending page number to include in the output PDF.')
    parser.add_argument('--page-step', type=int, default=1, metavar='STEP', help='Specify the step value when iterating through pages.')

    parser.add_argument('--user-style', default=str(), metavar='STYLE', help='Specify the user-defined CSS style to apply to the input document.')
    parser.add_argument('--user-script', default=str(), metavar='SCRIPT', help='Specify the user-defined JavaScript to run after the document has fully loaded.')

    parser.add_argument('--title', help='Set PDF document title.')
    parser.add_argument('--subject', help='Set PDF document subject.')
    parser.add_argument('--author', help='Set PDF document author.')
    parser.add_argument('--keywords', help='Set PDF document keywords.')
    parser.add_argument('--creator', help='Set PDF document creator.')

    parser.add_argument('--ssl-cainfo', metavar='FILE', type=str, help='Specify an SSL CA certificate file.')
    parser.add_argument('--ssl-capath', metavar='PATH', type=str, help='Specify an SSL CA certificate directory.')
    parser.add_argument('--no-ssl', action='store_true', help='Disable SSL verification (not recommended).')
    parser.add_argument('--no-redirects', action='store_true', help='Disable following HTTP redirects.')
    parser.add_argument('--max-redirects', metavar='NUM', type=int, help='Specify the maximum number of HTTP redirects to follow.')
    parser.add_argument('--timeout', metavar='SECONDS', type=int, help='Specify the HTTP timeout in seconds.')

    parser.add_argument('--version', action='version', version=f'PlutoPrint v{plutoprint.__version__}', help='Show version information and exit.')
    parser.add_argument('--info', action=InfoAction, nargs=0, help='Show build information and exit.')

    args = parser.parse_args()

    size = plutoprint.PAGE_SIZE_A4
    if args.size is not None:
        size = PAGE_SIZES[args.size]
    if args.width is not None:
        size = plutoprint.PageSize(args.width, size.height)
    if args.height is not None:
        size = plutoprint.PageSize(size.width, args.height)

    if args.orientation == 'portrait':
        size = size.portrait()
    elif args.orientation == 'landscape':
        size = size.landscape()

    margins = plutoprint.PAGE_MARGINS_NORMAL
    if args.margin is not None:
        margins = plutoprint.PageMargins(args.margin)
    if args.margin_top is not None:
        margins = plutoprint.PageMargins(args.margin_top, margins.right, margins.bottom, margins.left)
    if args.margin_right is not None:
        margins = plutoprint.PageMargins(margins.top, args.margin_right, margins.bottom, margins.left)
    if args.margin_bottom is not None:
        margins = plutoprint.PageMargins(margins.top, margins.right, args.margin_bottom, margins.left)
    if args.margin_left is not None:
        margins = plutoprint.PageMargins(margins.top, margins.right, margins.bottom, args.margin_left)

    media = plutoprint.MEDIA_TYPE_PRINT
    if args.media == 'screen':
        media = plutoprint.MEDIA_TYPE_SCREEN

    if args.ssl_cainfo is not None:
        plutoprint.default_resource_fetcher.set_ssl_cainfo(args.ssl_cainfo)
    if args.ssl_capath is not None:
        plutoprint.default_resource_fetcher.set_ssl_capath(args.ssl_capath)

    if args.no_ssl:
        plutoprint.default_resource_fetcher.set_ssl_verify_peer(False)
        plutoprint.default_resource_fetcher.set_ssl_verify_host(False)

    if args.no_redirects:
        plutoprint.default_resource_fetcher.set_http_follow_redirects(False)

    if args.max_redirects is not None:
        plutoprint.default_resource_fetcher.set_http_max_redirects(args.max_redirects)
    if args.timeout is not None:
        plutoprint.default_resource_fetcher.set_http_timeout(args.timeout)

    book = plutoprint.Book(size, margins, media)
    book.load_url(args.input, args.user_style, args.user_script)

    book.set_metadata(plutoprint.PDF_METADATA_CREATION_DATE, datetime.now().astimezone().isoformat(timespec='seconds'))
    for name, meta in METADATA_MAP.items():
        value = getattr(args, name)
        if value:
            book.set_metadata(meta, value)

    book.write_to_pdf(args.output, args.page_start, args.page_end, args.page_step)

if __name__ == '__main__':
    main()



import plutoprint
import pytest

def test_pagemargins_new():
    assert plutoprint.PageMargins()           == plutoprint.PageMargins(0, 0, 0, 0)
    assert plutoprint.PageMargins(1)          == plutoprint.PageMargins(1, 1, 1, 1)
    assert plutoprint.PageMargins(1, 2)       == plutoprint.PageMargins(1, 2, 1, 2)
    assert plutoprint.PageMargins(1, 2, 3)    == plutoprint.PageMargins(1, 2, 3, 2)
    assert plutoprint.PageMargins(1, 2, 3, 4) == plutoprint.PageMargins(1, 2, 3, 4)

    with pytest.raises(TypeError):
        plutoprint.PageMargins(1, 2, 3, 4, 5)

def test_pagemargins_repr():
    assert repr(plutoprint.PageMargins())           == "plutoprint.PageMargins(0, 0, 0, 0)"
    assert repr(plutoprint.PageMargins(1))          == "plutoprint.PageMargins(1, 1, 1, 1)"
    assert repr(plutoprint.PageMargins(1, 2))       == "plutoprint.PageMargins(1, 2, 1, 2)"
    assert repr(plutoprint.PageMargins(1, 2, 3))    == "plutoprint.PageMargins(1, 2, 3, 2)"
    assert repr(plutoprint.PageMargins(1, 2, 3, 4)) == "plutoprint.PageMargins(1, 2, 3, 4)"

def test_pagemargins_sequence():
    margins = plutoprint.PageMargins(1, 2, 3, 4)

    assert margins[0] == 1
    assert margins[1] == 2
    assert margins[2] == 3
    assert margins[3] == 4

    with pytest.raises(IndexError):
        margins[4]

    assert len(plutoprint.PageMargins())           == 4
    assert len(plutoprint.PageMargins(1))          == 4
    assert len(plutoprint.PageMargins(1, 2))       == 4
    assert len(plutoprint.PageMargins(1, 2, 3))    == 4
    assert len(plutoprint.PageMargins(1, 2, 3, 4)) == 4

def test_pagemargins_richcompare():
    a = plutoprint.PageMargins(1, 2, 3, 4)
    b = plutoprint.PageMargins(1, 2, 3, 4)
    c = plutoprint.PageMargins(4, 3, 2, 1)

    assert a == b
    assert a != c

    with pytest.raises(TypeError):
        a < b
    with pytest.raises(TypeError):
        a <= b
    with pytest.raises(TypeError):
        a > b
    with pytest.raises(TypeError):
        a >= b

def test_pagemargins_members():
    margins = plutoprint.PageMargins(1, 2, 3, 4)

    assert margins.top    == 1
    assert margins.right  == 2
    assert margins.bottom == 3
    assert margins.left   == 4

    with pytest.raises(AttributeError):
        margins.top = 5
    with pytest.raises(AttributeError):
        margins.right = 6
    with pytest.raises(AttributeError):
        margins.bottom = 7
    with pytest.raises(AttributeError):
        margins.left = 8

def test_pagemargins_constants():
    assert plutoprint.PAGE_MARGINS_NONE == plutoprint.PageMargins(0, 0, 0, 0)

    assert plutoprint.PAGE_MARGINS_NORMAL == plutoprint.PageMargins(72, 72, 72, 72)
    assert plutoprint.PAGE_MARGINS_NARROW == plutoprint.PageMargins(36, 36, 36, 36)

    assert plutoprint.PAGE_MARGINS_MODERATE == plutoprint.PageMargins(72, 54, 72, 54)
    assert plutoprint.PAGE_MARGINS_WIDE     == plutoprint.PageMargins(72, 144, 72, 144)



import plutoprint
import pytest

def test_mediatype():
    with pytest.raises(TypeError):
        plutoprint.MediaType()

    with pytest.raises(TypeError):
        plutoprint.MediaType(int())

    assert isinstance(plutoprint.MEDIA_TYPE_PRINT,  plutoprint.MediaType)
    assert isinstance(plutoprint.MEDIA_TYPE_SCREEN, plutoprint.MediaType)

    assert repr(plutoprint.MEDIA_TYPE_PRINT)  == "plutoprint.MEDIA_TYPE_PRINT"
    assert repr(plutoprint.MEDIA_TYPE_SCREEN) == "plutoprint.MEDIA_TYPE_SCREEN"

def test_pdfmetadata():
    with pytest.raises(TypeError):
        plutoprint.PDFMetadata()

    with pytest.raises(TypeError):
        plutoprint.PDFMetadata(int())

    assert isinstance(plutoprint.PDF_METADATA_TITLE,             plutoprint.PDFMetadata)
    assert isinstance(plutoprint.PDF_METADATA_AUTHOR,            plutoprint.PDFMetadata)
    assert isinstance(plutoprint.PDF_METADATA_SUBJECT,           plutoprint.PDFMetadata)
    assert isinstance(plutoprint.PDF_METADATA_KEYWORDS,          plutoprint.PDFMetadata)
    assert isinstance(plutoprint.PDF_METADATA_CREATOR,           plutoprint.PDFMetadata)
    assert isinstance(plutoprint.PDF_METADATA_CREATION_DATE,     plutoprint.PDFMetadata)
    assert isinstance(plutoprint.PDF_METADATA_MODIFICATION_DATE, plutoprint.PDFMetadata)

    assert repr(plutoprint.PDF_METADATA_TITLE)             == "plutoprint.PDF_METADATA_TITLE"
    assert repr(plutoprint.PDF_METADATA_AUTHOR)            == "plutoprint.PDF_METADATA_AUTHOR"
    assert repr(plutoprint.PDF_METADATA_SUBJECT)           == "plutoprint.PDF_METADATA_SUBJECT"
    assert repr(plutoprint.PDF_METADATA_KEYWORDS)          == "plutoprint.PDF_METADATA_KEYWORDS"
    assert repr(plutoprint.PDF_METADATA_CREATOR)           == "plutoprint.PDF_METADATA_CREATOR"
    assert repr(plutoprint.PDF_METADATA_CREATION_DATE)     == "plutoprint.PDF_METADATA_CREATION_DATE"
    assert repr(plutoprint.PDF_METADATA_MODIFICATION_DATE) == "plutoprint.PDF_METADATA_MODIFICATION_DATE"

def test_imageformat():
    with pytest.raises(TypeError):
        plutoprint.ImageFormat()

    with pytest.raises(TypeError):
        plutoprint.ImageFormat(int())

    assert isinstance(plutoprint.IMAGE_FORMAT_INVALID, plutoprint.ImageFormat)
    assert isinstance(plutoprint.IMAGE_FORMAT_ARGB32,  plutoprint.ImageFormat)
    assert isinstance(plutoprint.IMAGE_FORMAT_RGB24,   plutoprint.ImageFormat)
    assert isinstance(plutoprint.IMAGE_FORMAT_A8,      plutoprint.ImageFormat)
    assert isinstance(plutoprint.IMAGE_FORMAT_A1,      plutoprint.ImageFormat)

    assert repr(plutoprint.IMAGE_FORMAT_INVALID) == "plutoprint.IMAGE_FORMAT_INVALID"
    assert repr(plutoprint.IMAGE_FORMAT_ARGB32)  == "plutoprint.IMAGE_FORMAT_ARGB32"
    assert repr(plutoprint.IMAGE_FORMAT_RGB24)   == "plutoprint.IMAGE_FORMAT_RGB24"
    assert repr(plutoprint.IMAGE_FORMAT_A8)      == "plutoprint.IMAGE_FORMAT_A8"
    assert repr(plutoprint.IMAGE_FORMAT_A1)      == "plutoprint.IMAGE_FORMAT_A1"



import plutoprint
import pytest

def test_pagesize_new():
    assert plutoprint.PageSize()     == plutoprint.PageSize(0, 0)
    assert plutoprint.PageSize(1)    == plutoprint.PageSize(1, 1)
    assert plutoprint.PageSize(1, 2) == plutoprint.PageSize(1, 2)

    with pytest.raises(TypeError):
        plutoprint.PageSize(1, 2, 3)

def test_pagesize_repr():
    assert repr(plutoprint.PageSize())     == "plutoprint.PageSize(0, 0)"
    assert repr(plutoprint.PageSize(1))    == "plutoprint.PageSize(1, 1)"
    assert repr(plutoprint.PageSize(1, 2)) == "plutoprint.PageSize(1, 2)"

def test_pagesize_landscape():
    assert plutoprint.PageSize(1, 2).landscape() == plutoprint.PageSize(2, 1)
    assert plutoprint.PageSize(2, 1).landscape() == plutoprint.PageSize(2, 1)
    assert plutoprint.PageSize(2, 2).landscape() == plutoprint.PageSize(2, 2)

def test_pagesize_portrait():
    assert plutoprint.PageSize(2, 1).portrait() == plutoprint.PageSize(1, 2)
    assert plutoprint.PageSize(1, 2).portrait() == plutoprint.PageSize(1, 2)
    assert plutoprint.PageSize(2, 2).portrait() == plutoprint.PageSize(2, 2)

def test_pagesize_sequence():
    size = plutoprint.PageSize(1, 2)

    assert size[0] == 1
    assert size[1] == 2

    with pytest.raises(IndexError):
        size[2]

    assert len(plutoprint.PageSize())     == 2
    assert len(plutoprint.PageSize(1))    == 2
    assert len(plutoprint.PageSize(1, 2)) == 2

def test_pagesize_richcompare():
    a = plutoprint.PageSize(1, 2)
    b = plutoprint.PageSize(1, 2)
    c = plutoprint.PageSize(2, 1)

    assert a == b
    assert a != c

    with pytest.raises(TypeError):
        a < b
    with pytest.raises(TypeError):
        a <= b
    with pytest.raises(TypeError):
        a > b
    with pytest.raises(TypeError):
        a >= b

def test_pagesize_members():
    size = plutoprint.PageSize(1, 2)

    assert size.width  == 1
    assert size.height == 2

    with pytest.raises(AttributeError):
        size.width = 3
    with pytest.raises(AttributeError):
        size.height = 4

def test_pagesize_constants():
    assert plutoprint.PAGE_SIZE_NONE == plutoprint.PageSize()

    assert plutoprint.PAGE_SIZE_LETTER == plutoprint.PageSize(8.5 * plutoprint.UNITS_IN, 11 * plutoprint.UNITS_IN)
    assert plutoprint.PAGE_SIZE_LEGAL  == plutoprint.PageSize(8.5 * plutoprint.UNITS_IN, 14 * plutoprint.UNITS_IN)
    assert plutoprint.PAGE_SIZE_LEDGER == plutoprint.PageSize(11 * plutoprint.UNITS_IN, 17 * plutoprint.UNITS_IN)

    assert plutoprint.PAGE_SIZE_A3 == plutoprint.PageSize(297 * plutoprint.UNITS_MM, 420 * plutoprint.UNITS_MM)
    assert plutoprint.PAGE_SIZE_A4 == plutoprint.PageSize(210 * plutoprint.UNITS_MM, 297 * plutoprint.UNITS_MM)
    assert plutoprint.PAGE_SIZE_A5 == plutoprint.PageSize(148 * plutoprint.UNITS_MM, 210 * plutoprint.UNITS_MM)

    assert plutoprint.PAGE_SIZE_B4 == plutoprint.PageSize(250 * plutoprint.UNITS_MM, 353 * plutoprint.UNITS_MM)
    assert plutoprint.PAGE_SIZE_B5 == plutoprint.PageSize(176 * plutoprint.UNITS_MM, 250 * plutoprint.UNITS_MM)
